﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Routines;
using Styx.Helpers;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.MemoryManagement;
using Action = Styx.TreeSharp.Action;


namespace Tricks
{
    public partial class Routine
    {

        
        public IEnumerable<WoWUnit> EnemiesNeedingKick
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.IsWithinMeleeRange &&
                            u.IsCasting &&
                            u.CurrentCastTimeLeft.TotalMilliseconds < CastTimeLeftInMS
                            //u.CastingSpell.Name == "Fear" ||
                            //u.CastingSpell.Name == "Polymorph" ||
                            //u.CastingSpell.Name == "Cyclone" ||
                            //u.CastingSpell.Name == "Repentence" ||
                            //u.CastingSpell.Name == "Hex" ||
                            //u.IsCastingHealingSpell
                                  );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> nearbyFriendliesBeasting
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        p =>

                        p.Distance <= 100
                        && !p.IsMe
                        && p.IsInMyPartyOrRaid
                        && p.InLineOfSight
                        && IsBeasting(p));

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWPlayer> nearbyFriendliesNeedingTricks
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        p =>
                        p.Distance <= 100
                        && !p.IsMe
                        && p.IsInMyPartyOrRaid
                        && p.InLineOfSight
                        //&& 
                        
                        //IsBursting(p) || 
                        //IsBeasting(p) || 
                        //(p.IsCasting && HS_TrickSpellIDs.Contains(p.CastingSpell.Id))

                        );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWPlayer> nearbyFriendliesCasting
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        p =>
                        p.IsCasting
                        && !p.IsMe
                        && p.Distance <= 100
                        && p.IsInMyPartyOrRaid
                        && p.InLineOfSight
                        && HS_TrickSpellIDs.Contains(p.CastingSpell.Id));
                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemiesNeedingGougeInterrupt
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.IsWithinMeleeRange &&
                            StyxWoW.Me.IsFacing(u) &&
                            u.IsCasting &&
                            u.CurrentCastTimeLeft.TotalMilliseconds < CastTimeLeftInMS && (
                            u.CastingSpell.Name == "Fear" ||
                            u.CastingSpell.Name == "Polymorph" ||
                            u.CastingSpell.Name == "Cyclone" ||
                            u.CastingSpell.Name == "Repentence" ||
                            u.CastingSpell.Name == "Hex" ||
                            u.IsCastingHealingSpell
                            )
                                  );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> FindStealthy
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            !u.IsMe &&
                            !u.IsPet &&
                            !u.Combat &&
                            (u.IsHumanoid || u.IsBeast || u.IsDemon || u.IsDragon) &&
                            u.Distance <= 10 &&
                            u.IsStealthed

                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemiesTargetingMe
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsHostile &&
                            u.Distance <= 40 &&
                            u.IsAlive &&
                            !u.IsMe &&
                            !u.IsPet &&
                            u.IsTargetingMeOrPet
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> FriendsNeedingHelp30
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsFriendly &&
                            u.HealthPercent <= 25 &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 30
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWPlayer> NeedsTricks
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            !u.IsMe &&
                            u.IsInMyPartyOrRaid &&
                            u.Distance <= 100 &&
                            u.InLineOfSpellSight &&
                            u.IsCasting &&
                            (u.CastingSpell.Name == "Mind Blast")

                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> LastSapTargetCheck
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u == LastSapTarget &&
                            IsAttackable(u) &&
                            u.Distance <= 15 &&
                            u.InLineOfSight
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public bool DontBreakCC(WoWUnit unit)
        {
            if (unit != null)
            {
                var auraList = unit.GetAllAuras();

                foreach (WoWAura aura in auraList)
                {
                    if (!aura.IsActive)
                        continue;
                    if (HS_DontBreakCC.Contains(aura.SpellId))
                        return true;
                }
            }
            return false;
        }
        public bool HasDOTs(WoWUnit unit)
        {
            return
                unit.ActiveAuras.Any(a =>
                    a.Value.SpellId != 57724 && //Sated
                    a.Value.SpellId != 80354 && //Temporal Displacement
					a.Value.SpellId != 25771 && //Forbearance
                    (a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicDamage ||
                    a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicDamagePercent ||
                    a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicLeech)
                );
        }
        public bool IsAttackable(WoWUnit unit)
        {
            if (unit == null)
                return false;

            if (unit == StyxWoW.Me)
                return false;

            if (IsImmune(unit))
                return false;

            if (DontBreakCC(unit))
                return false;

            if (!unit.CanSelect || !unit.Attackable)
                return false;

            if (unit.IsDead)
                return false;

            if (unit.IsOnTransport)
                return false;

            //if (unit.IsHostile || !unit.IsFriendly)
            //    return true;

            return true;
        }
        public bool IsBeasting(WoWPlayer unit)
        {

            var auraList = unit.GetAllAuras();

            foreach (WoWAura aura in auraList)
            {
                if (!aura.IsActive)
                    continue;
                if (HS_IsBeasting.Contains(aura.ApplyAuraType))
                    return true;
            }
            return false;
        }
        public bool IsBursting(WoWUnit unit)
        {
            if (unit != null)
            {
                var auraList = unit.GetAllAuras();

                foreach (WoWAura aura in auraList)
                {
                    if (!aura.IsActive)
                        continue;
                    if (HS_BurstingIDs.Contains(aura.Spell.Id))
                        return true;
                }

            }
            return false;
        }
        public bool IsCaster(WoWUnit unit)
        {
            if (unit != null)
            {

                if (
                    unit.Class == WoWClass.Priest ||
                    (unit.Class == WoWClass.Druid && unit.CurrentMana >= 100000) ||
                    (unit.Class == WoWClass.Monk && unit.CurrentMana >= 100000) ||
                    unit.Class == WoWClass.Mage ||
                    unit.Class == WoWClass.Warlock ||
                    (unit.Class == WoWClass.Paladin && unit.CurrentMana >= 100000) ||
                    (unit.Class == WoWClass.Shaman && unit.CurrentMana >= 100000)
                    )
                return true;
            }
            return false;
        }
        public bool IsImmune(WoWUnit unit)
                {
                    if (unit != null)
                    {
                        var auraList = unit.GetAllAuras();

   

                        foreach (WoWAura aura in auraList)
                        {
                            if (!aura.IsActive)
                                continue;
                            if (HS_ImmuneDebuffIDs.Contains(aura.SpellId))
                                return true;
                        }
                    }
                    return false;
                }
        public bool IsInCC(WoWUnit unit)
            {
                if (unit != null)
                {
                    if (unit.ActiveAuras.ContainsKey("Repentance") ||
                        unit.ActiveAuras.ContainsKey("Blinding Light") ||
                        unit.ActiveAuras.ContainsKey("Intimidating Shout") ||
                        unit.ActiveAuras.ContainsKey("Wyvern Sting") ||
                        unit.ActiveAuras.ContainsKey("Polymorph") ||
                        unit.ActiveAuras.ContainsKey("Freezing Trap") ||
                        unit.ActiveAuras.ContainsKey("Sap") ||
                        unit.ActiveAuras.ContainsKey("Blind"))
                        return true;
                }
                return false;
            }
        public bool IsCrowdControlled(WoWUnit unit)
        {
            if (unit != null)
            {
                var auraList = unit.GetAllAuras();

                foreach (WoWAura aura in auraList)
                {
                    if (!aura.IsActive)
                        continue;
                    if (HS_CCMechanics.Contains(aura.Spell.Mechanic))
                        return true;
                }
            }
            return false;
        }
        public bool IsInterruptImmune(WoWUnit unit)
        {
            if (unit != null)
            {
                var auraList = unit.GetAllAuras();

                foreach (WoWAura aura in auraList)
                {
                    if (!aura.IsActive)
                        continue;
                    if (InterruptImmuneAuraIDs.Contains(aura.SpellId))
                        return true;
                }
            }
            return false;
        }
        public bool IsMovementImpaired(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (a.Spell.Mechanic == WoWSpellMechanic.Snared || a.Spell.Mechanic == WoWSpellMechanic.Rooted));
            }
            return false;
        }
        public bool IsStunned(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (
                           a.Spell.Mechanic == WoWSpellMechanic.Stunned));
            }
            return false;
        }
        public int MyDebuffTime(String spellName, WoWUnit unit)
        {
            using (StyxWoW.Memory.AcquireFrame())
            {
                if (unit.HasAura(spellName))
                {
                    var auras = unit.GetAllAuras();
                    foreach (var a in auras)
                    {
                        if (a.Name == spellName && a.CreatorGuid == StyxWoW.Me.Guid)
                        {
                            return a.TimeLeft.Seconds;
                        }
                    }
                }
            }
            return 0;
        }
        public bool NeedsDismantle(WoWUnit unit)
        {
            if (unit != null)
            {
                var auraList = unit.GetAllAuras();

                foreach (WoWAura aura in auraList)
                {
                    if (!aura.IsActive)
                        continue;
                    if (HS_BurstingIDs.Contains(aura.Spell.Id))
                        return true;
                }
                
            }
          return false;
        }
        public static string RealLuaEscape(string luastring)
        {
            var bytes = Encoding.UTF8.GetBytes(luastring);
            return bytes.Aggregate(String.Empty, (current, b) => current + ("\\" + b));
        }
        public bool ShouldSap
        {
            get
            {
                IEnumerable<WoWPlayer> EnemyUnitsInSap = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
                .Where(u =>
                    IsAttackable(u) &&
                    u.ActiveAuras.Any(a =>
                        a.Value.SpellId == 6770 &&
                        a.Value.CreatorGuid == StyxWoW.Me.Guid));

                if (EnemyUnitsInSap.Count() >= 1)
                {
                    return false;
                }

                return true;
            }
        }
        #region Tricks

        static string[] CrowdControlSpellNames = new string[] { "Chaos Bolt", "Cyclone", "Entangling Roots", "Fear", "Hex", "Polymorph" };

        static IEnumerable<WoWPlayer> EnemiesCastingCrowdControl60 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 60 && p.IsHostile && p.IsCasting && CrowdControlSpellNames.Contains(p.CastingSpell.Name)).ToList();

        static IEnumerable<WoWPlayer> nearbyFriendlies20 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 20 && p.IsFriendly && p.IsAlive).ToList();

        IEnumerable<WoWApplyAuraType> _dpsIncreaseAurasTypes = new List<WoWApplyAuraType>()
        {
            WoWApplyAuraType.ModAllCritChance,
            WoWApplyAuraType.ModAttackerMeleeCritChance,
            WoWApplyAuraType.ModAttackerRangedCritChance,
            WoWApplyAuraType.ModAttackerSpellAndWeaponCritChance,
            WoWApplyAuraType.ModAttackerSpellCritChance,
            WoWApplyAuraType.ModCritPercent,
            WoWApplyAuraType.ModCritDamageBonusMelee,
            WoWApplyAuraType.ModDamageDone,
            WoWApplyAuraType.ModDamagePercentDone,
            WoWApplyAuraType.ModSpellCritChance,
            WoWApplyAuraType.ModDamagePercentDone,
            WoWApplyAuraType.ModAttackPower,
            WoWApplyAuraType.ModAttackPowerPct
            
        };
        #endregion





    }
}
